---
title: "Mini-Tutorial: LEA"
subtitle: Kurze Anleitung zur Arbeit mit dem Konsolenprogramm LEA
date: "2022-10-30 (letzte Änderung: 2022-11-06)"
author: Stefan Reiser <s.reiser@tu-braunschweig.de>
---

# Einleitung

LEA ist ein reines Konsolenprogramm: Es nimmt *Eingabedateien* (Berechnungsjobs)
entgegen und produziert *Ausgabedateien* (Ergebnisse).
Solche Programme sind heute für viele völlig
ungewohnt. Das vorliegende Tutorial hilft, sich mit der Bedienung
vertraut zu machen.

Das Tutorial wurde unter Windows erstellt.
Mit geringfügigen Änderungen lassen sich alle Schritte auch auf anderen
Betriebssystemen durchführen. Siehe hierzu die
[Hinweise für Unix/Linux/MacOS](#unix) im Anhang.

# Vorbereitungen

## Installation auf einen USB-Stick

Das Programm kann in einen beliebigen Ordner installiert werden. In
diesem Beispiel wird ein USB-Stick mit dem Laufwerksbuchstaben
"G:" verwendet.

* Laden Sie die [aktuelle ZIP-Datei
lea-20xx-xx-xx.zip](http://suse.straw.bau.tu-bs.de/software/#lea) herunter.
* Speichern Sie die Datei auf Ihrem USB-Stick und entpacken Sie sie dort.
* Benennen Sie den Ordner, in den entpackt wurde (der vermutlich
 "lea-20xx-xx-xx" heißt), in "lea" um.

Der Inhalt sollte wie folgt aussehen:

![](./img/exp-lea.png)

Der Unterordner "bin" enthält Programme, "doc" die Dokumentation. In
"work" legen Sie Ihre Berechnungen ab.

## Weitere Hilfsmittel

* Es muss eine **Java-Laufzeitumgebung (JRE)** installiert sein. Diese
  ist unter [www.java.com](https://www.java.com) erhältlich.
* Ein vernünftiger **Texteditor** ist empfehlenswert. Geeignete
  Kandidaten sind [Notepad++], [VS Code] und [Programmer's Notebook].
* Für die Arbeit mit CSV-Dateien empfielt sich "Calc" von [LibreOffice].
  (Siehe die [Anmerkungen dazu im Anhang](#lo).)

# Eine Berechnung durchführen

Öffnen Sie den Unterordner "work" mit dem Windows-Explorer.

![](./img/exp-work.png)

Gehen Sie dann in die Adresszeile (Tastenkombination
<kbd>Strg</kbd>+<kbd>L</kbd>). Dort tippen Sie "cmd" und drücken
<kbd>Enter</kbd>, um eine Befehlskonsole in diesem Ordner zu öffnen.

![](./img/cmd.png)

Geben Sie den Befehl `java -version`{.hl} ein, um zu prüfen, ob Java korrekt
installiert wurde. Das ist der Fall, wenn Sie eine Meldung wie die oben
abgebildete erhalten. (&#x26A0; Lesen Sie die [Hinweise zur Bedienung der
Befehlskonsole](#cmd2) und nehmen Sie die [empfohlenen Einstellungen](#cmd1)
vor.)

Öffnen Sie die Datei "beispiel.txt" im Editor. Es handelt sich um eine
ausführlich kommentierte Eingabedatei, die Sie als Grundlage für eigene
Berechnungen verwenden können.

Wir möchten diese Eingabedatei von LEA abarbeiten lassen:
Lassen Sie sich zunächst mit dem Aufrufparameter `-h` den Hilfetext
des Programms anzeigen.
Befehl: `java -jar ..\bin\leaapp\JavaApplicationLea.jar -h`{.hl}. Dies
liefert eine Übersicht der möglichen Aufrufparameter und ihrer Bedeutung.

Starten Sie jetzt die Berechnung:
`java -jar ..\bin\leaapp\JavaApplicationLea.jar -C -i beispiel.txt -o ergebnis.csv`{.hl}.
Im work-Ordner sollte eine neue Datei namens "ergebnis.csv" entstanden sein.

>*Erläuterungen:* Mit `-i`{.hl} wurde "beispiel.txt" als Eingabedatei festgelegt,
mit `-o`{.hl} die Ausgabedatei "ergebnis.csv".
Der Parameter `-C`{.hl} sagt LEA, dass es seine Konfiguration direkt aus
der Eingabedatei lesen soll (siehe Zeile&nbsp;86, Config-Block).

### Einige Experimente/Umgang mit Fehlermeldungen:

Wenn Sie `-o ergebnis.csv`{.hl} weglassen, werden die Ergebnisse ins
Konsolenfenster geschrieben statt in eine Ausgabedatei.
Lassen Sie versuchsweise den Parameter `-C`{.hl} weg.
Das Programm endet mit der Fehlermeldung

    Error loading output points.
    'outpoint' expected.
    Line: 'config'

Die Meldung besagt, dass der Abbruch stattfand, als das Programm die
Ausgabepunkte laden wollte. Es erwartete an dieser Stelle in der
Eingabedatei das Schlüsselwort `outpoint`{.hl}, fand jedoch eine Zeile
mit dem Inhalt '`config`'.

Dies hilft Ihnen, den Fehler aufzuspüren:
Suchen Sie im Texteditor die fragliche Stelle auf -- den config-Block
ab Zeile 86. Sie beheben das Problem, indem Sie diesen "auskommentieren",
d.h., Sie setzen ein "#" vor jede Zeile des Blocks.
Nach dem Speichern führen Sie den letzen Befehl erneut aus.
Das Programm sollte nun erfolgreich durchlaufen.

## Ergebnisse ansehen
Öffnen Sie die Datei "ergebnis.csv" in LibreOffice Calc (per Rechtsklick
auf die Datei -- oder legen Sie fest, dass .csv-Dateien grundsätzlich
mit Calc geöffnet werden). Es erscheint ein Import-Dialog, in dem Sie
folgende Angaben machen: Sprache 'Englisch', Trennzeichen 'Tabulator',
Zeichenketten-Trenner '"'. Der Import-Dialog merkt sich die zuletzt
verwendeten Einstellungen.

![](./img/calc-csvimport.png)

![](./img/calc.png)

Die Felder sind:
*disp* Verschiebung (Displacement), *stress* Spannung, *strain* Dehnung.
Weitere Informationen, speziell zu den Hauptspannungen, finden Sie in
der Dokumentation in "lea2doc.pdf" im Abschnitt&nbsp;5,&nbsp;CSV-Ausgabe. Siehe auch die
Anmerkungen zu Maßeinheiten in "beispiel.txt".

## Ein weiteres Berechnungsbeispiel
Wir möchten den Spannungsverlauf an der Oberfläche ermitteln.
Dazu werden Ausgabepunkte im 1-mm-Abstand entlang der x-Achse gesetzt,
vom Ursprung (0,&nbsp;0,&nbsp;0) bis zur Koordinate (2,&nbsp;0,&nbsp;0).
Angaben in&nbsp;m.

Es werden also 2000 Ausgabepunkte gebraucht, die niemand von
Hand eintippen möchte.
 Nehmen Sie ein neues Tabellenblatt in Calc.
Geben Sie die Zeilen 2--5 wie in der folgenden Abbildung ein. (Beachten
Sie, dass ein [Dezimalpunkt statt eines Kommas](#lo-lang) verwendet wird.)

>&#x26A0;&nbsp;Achtung: Bitte geben Sie **nach dem Wort '`add`' ein Leerzeichen**
ein, wenn Sie eine ältere LEA-Version nutzen. Sie umgehen damit
einen [&#x1F41E; Bug in LEA](#bug), der beim Erstellen
des Tutorials aufgefallen ist. Der Programmfehler wurde in Version
2022-11-06 behoben.

![](./img/calc-outpoint.png)

Markieren Sie nun wie abgebildet die Felder der Zeilen 4 und 5. Ziehen
Sie das schwarze Kästchen an der rechten unteren Ecke der Markierung mit
der Maus nach unten, um die
[Auto-Ausfüllen-Funktion][lo-autofill]
aufzurufen.
Lassen Sie die Maustaste los, wenn Sie ungefähr in Zeile 2000 angekommen
sind -- auf einige Zeilen mehr oder weniger kommt es nicht an.

Markieren Sie alles mit <kbd>Strg</kbd>+<kbd>A</kbd>, kopieren
Sie mit <kbd>Strg</kbd>+<kbd>C</kbd> und fügen Sie die Daten
mit <kbd>Strg</kbd>+<kbd>V</kbd> im Texteditor
an der Stelle des bisherigen Outpoint-Blocks ein. Speichern Sie die Datei
als "beispiel2.txt".

![Texteditor mit aus Calc eingefügten Daten.
Tabs und Leerzeichen sind sichtbar gemacht.](./img/edit-outpoint.png)

Führen Sie abschließend die Berechnung durch:\
`java -jar ..\bin\leaapp\JavaApplicationLea.jar -i beispiel2.txt -o ergebnis2.csv`{.hl}


## Ergebnisse plotten
Gehen Sie mit dem Windows-Explorer ins Verzeichnis "G:\\bin\\gnuplot\\bin".
Starten Sie dort das Programm "wgnuplot.exe".
Machen Sie mit "ChDir" in der Symbolleiste das Verzeichnis "work" zu Gnuplots Arbeitsverzeichnis:

![](./img/gp.png)\
Alternativ hätten Sie den Befehl `cd 'G:\lea\work'`{.hl} eintippen können.

Nehmen Sie einige grundlegende Einstellungen für die Plots vor:

~~~~~~~~~~~~~~~~~
set grid
set style data lines
set key center
set xlabel "x [m]"
~~~~~~~~~~~~~~~~~

>Einfügen aus der Zwischenablage geht in Gnuplot mit
<kbd>Umschalt</kbd>+<kbd>Einfg</kbd> oder Rechtsklick, 'Paste'. Sie
können mehrere Befehle auf einmal einfügen. Eventuell müssen Sie
Enter drücken, um den letzten Befehl abzuschließen.

Erzeugen Sie nacheinander diese drei Plots für Verschiebungen, Spannungen und Dehnungen:

~~~~~~~~~~~~~~~~~
set ylabel "Meter"
plot "ergebnis2.csv" using 3:7 title column, "" us 3:8 t col, "" us 3:9 t col

set ylabel "MPa"
plot "ergebnis2.csv" us 3:10 t col, "" us 3:11 t col, "" us 3:12 t col

set ylabel "Meter/Meter"
plot "ergebnis2.csv" us 3:22 t col, "" us 3:23 t col, "" us 3:24 t col
~~~~~~~~~~~~~~~~~

Das Ergebnis sollte etwa so aussehen:
<div style="text-align:center">
![](./img/gp-disp.png)
![](./img/gp-stress.png)
![](./img/gp-strain.png)
</div>

>Erläuterungen zu &#x1F5E0;[Gnuplot] sprengen hier den Rahmen. Bei
Interesse finden Sie ausführliche Anleitungen und Beispiele auf der
Homepage des Programms.

# Anhang

## Empfohlene Einstellungen für die Befehlskonsole "cmd" {#cmd1}

**Einstellungen öffnen:**
Rechtsklick auf den Fenstertitel, "Eigenschaften".

Im Tab "Optionen" aktivieren Sie den *Quick Edit-Modus* und den
*Einfügemodus*. Im Tab "Layout" unter *Fenstergröße* setzen Sie die
Breite auf mindesten 80 Zeichen. Angenehme Werte für die Höhe sind
35--50 Zeilen.

![](./img/cmd-prop.png)

## Hinweise zur Bedienung der Befehlskonsole {#cmd2}

Eingabezeile:

* <kbd>Enter</kbd> führt den eingegebenen Befehl aus. <kbd>ESC</kbd>
  löscht die Eingabe.
* Mit <kbd>&larr;</kbd>, <kbd>&rarr;</kbd>, <kbd>Pos1</kbd> und
  <kbd>Ende</kbd> bewegen Sie sich in der Eingabezeile. (Der Cursor lässt
  sich nicht mit der Maus positionieren!)
  Mit <kbd>Backspace</kbd> und <kbd>Entf</kbd> wird wie gewohnt Text
  gelöscht.
* Mit <kbd>&uarr;</kbd> und <kbd>&darr;</kbd> blättern Sie durch die
  zuletzt eingegeben Befehle.
* <kbd>TAB</kbd> ergänzt Dateinamen. Tippen Sie den Anfang eines

  Dateinamens und schalten Sie durch wiederholtes Drücken der
  Tabulator-Taste durch die dazu passenden Dateien.

Zwischenablage:

* *Linke Maustaste:* drücken und halten, um Text zu markieren.
  Den Text mit <kbd>Enter</kbd> in die Zwischenablage kopieren
  oder das Markieren mit <kbd>ESC</kbd> abbrechen.
* *Rechte Maustaste:* fügt den Inhalt der Zwischenablage an der Cursorposition
  ein.
  Sie können sich Tipparbeit sparen, indem Sie Befehle des
  Tutorials kopieren und mit Rechtsklick in die Konsole einfügen.\
  &#x26A0;&nbsp;Achtung: Prüfen Sie genau, was Sie einfügen -- es wird als
  Befehl ausgeführt! (Insbesondere ergibt mehrzeiliger Text eine Folge
  von Befehlen.)

## Anmerkungen zu LibreOffice/Calc {#lo}
Mit [LibreOffice]/Calc ein zweites Spreadsheet-Programm
"in der Hinterhand" zu haben, lohnt sich auch für all jene, die primär
Excel benutzen, denn Excel ist notorisch schlecht im Umgang mit
Text- und CSV-Dateien.

Die meisten Programme, die numerische Berechnungen durchführen (so auch LEA),
verwenden Textdateien zum Datenaustausch und stellen Zahlen in
internationaler Schreibweise mit *Punkt als Dezimaltrennzeichen* dar.
Tabellenkalkulationen (Spreadsheets) andererseits richten ihre
Zahlendarstellung an den lokalen Spracheinstellungen aus.

Calc hat hier gegenüber Excel zwei entscheidende Vorteile:

1) Der beim Öffnen einer Text-/CSV-Datei erscheinende Import-Dialog ist
   mit einem Klick erledigt. Er merkt sich die letzten Einstellungen für das
   nächstes Mal.

   Excels vielschrittiger Import-Assistent muss stets aufs
   neue durchlaufen werden. (In jüngeren Office-365-Versionen ist er als
   *Legacy* eingestuft und standardmäßig deaktiviert. Siehe Dokumentation
   im [Original][excel-import-e] oder auf [Roboter-Deutsch][excel-import-d].)

2) Excel lässt sich nur über die systemweiten Ländereinstellungen
   (Systemsteuerung/Region) zum Verwenden des Dezimalpunkts überreden.
   Dortige Änderungen beeinflussen aber auch *sämtliche anderen Programme*, was
   selten erwünscht sein dürfte.
   In LibreOffice lassen sich Ländereinstellungen unabhängig
   konfigurieren ([siehe unten](#lo-lang)), während der Rest des Systems
   in Ruhe gelassen wird.

**Hinweis:** Der Datenaustausch zwischen Excel und Calc  über die
Zwischenablage funktioniert auch bei unterschiedlichen Ländereinstellungen
problemlos.


### Punkt als Dezimaltrennzeichen verwenden {#lo-lang}
Öffnen Sie das Menü *Extras*, *Optionen*.
Stellen Sie unter *Spracheinstellungen*, *Sprachen* im Abschnitt *Formate*
das Gebietsschema auf "English (USA)". Stellen Sie die übrigen Felder
wie unten abgebildet zurück auf "Deutsch (Deutschland)".\
&#x26A0;&nbsp;Feld&nbsp;*Datumserkennungsmuster* beachten!

![](./img/calc-optionen.png)

## Hinweise für Unix/Linux/MacOS {#unix}
* Diese Betriebssysteme verwenden für Pfadangaben einen *Slash* ("/")
an Stelle des unter Windows üblichen *Backslash* ("\\"). Passen Sie die Konsolenbefehle
entsprechend an.

* Für MacOS: [Anleitung zum Öffnen eines *Terminals* (Befehlskonsole) im
aktuellen Verzeichnis des *Finders*][finder-terminal].

* Java, Gnuplot und LibreOffice: Die Anwendungen sind für alle
Betriebssysteme verfügbar. Unter Linux bezieht man sie üblicherweise
über den Paketmanager der Distribution. Für MacOS findet man eine Version
auf den Webseiten des jeweiligen Programms
([Java](https://www.java.com), [Gnuplot][gnuplot-dl], [LibreOffice]).

## Bemerkungen zum Excel-Frontend {#excel}

Das im Ordner "bin\\excel" enthaltene Excel-Frontend leistet ungefähr
das, was Sie im vorliegenden Tutorial gelernt haben.
Das Frontend erzeugt Jobdateien, ruft LEA
auf, liest Ergebnisse ein und erstellt vergleichende Plots mit Gnuplot.
Es funktioniert unter Windows mit Excel 2000--2019
einschließlich 64-Bit-Office-Versionen.
Eine Anleitung befindet sich in der Datei
"doc\\liesmich!-excel.txt".

Das Excel-Frontend wurde ursprünglich für hausinterne Zwecke erstellt. Der
Versuch, es auch in der Lehre einzusetzen, führt zu stets
wiederkehrenden Problemen, die oft nur einige Teilnehmer betreffen,
für die Betroffenen jedoch nicht zu lösen sind. Von "ich habe gar kein Excel"
abgesehen sind das etwa die Verwendung von *MacOS*, zunehmend strenge
Sicherheitsvorgaben, die das Ausführen von
VBA-Programmen unterbinden, oder die Verwendung von *Office Online*, das
gar kein VBA unterstützt. Zur Abhilfe wurde dieses Tutorial erstellt.

## Bug in LEA {#bug}
Gilt für Versionen 2020-09-03 und älter. Der Programmfehler wurde mit
Version 2022-11-06 behoben.

&#x1F41E; **Beschreibung:**
Die Syntax der Eingabedatei erlaubt eine beliebige Vermischung
von Tabs und Leerzeichen (Whitespace). Beim *add-*Befehl in den
Blöcken *outpoint*, *load* und *struct* erwartet das Programm jedoch
(fälschlich) ein Leerzeichen hinter dem Wort "add". Wird ein Tab
vorgefunden, bricht es mit einer Fehlermeldung ab.

Das Problem tritt insbesondere beim Einfügen aus
Tabellenkalkulationen (Excel, Calc) in einen Texteditor auf. Man
erkennt es an Fehlermeldungen wie folgender, in
der das Programm einen Syntaxfehler bemängelt, obwohl die
Syntax eingehalten wurde:

    Error loading output points.
    'add <OUTPUTPOINT>' expected.
    Line: 'add     pt1     0.001   0       0'

**Workaround:** Entweder im Texteditor alle Tabs ("\\t") durch Leerzeichen
ersetzen lassen oder hinter dem Schlüsselwort "add" bewusst ein
überflüssiges Leerzeichen einfügen.


[Notepad++]: https://notepad-plus-plus.org/
[VS Code]: https://code.visualstudio.com/
[Programmer's Notebook]: https://www.pnotepad.org/
[LibreOffice]: https://www.libreoffice.org/
[excel-import-e]: https://support.microsoft.com/en-us/office/data-import-and-analysis-options-3ea52160-08bc-45ac-acd9-bc4a11bcc2a2
[excel-import-d]: https://support.microsoft.com/de-de/office/optionen-f%C3%BCr-den-datenimport-und-die-datenanalyse-3ea52160-08bc-45ac-acd9-bc4a11bcc2a2
[lo-autofill]: https://help.libreoffice.org/7.2/de/text/scalc/guide/calc_series.html?DbPAR=CALC#bm_id3150769
[finder-terminal]: https://apple.stackexchange.com/questions/11323/how-can-i-open-a-terminal-window-directly-from-my-current-finder-location
[Gnuplot]: http://gnuplot.info
[gnuplot-dl]: http://gnuplot.info/download.html